#include <avr/io.h>
#include <avr/pgmspace.h>
#include "compiler.h"
#include "usart.h"

#ifdef UDR0
#define UBRRH UBRR0H
#define UBRRL UBRR0L
#define UDR UDR0

#define UCSRA UCSR0A
#define UDRE UDRE0
#define RXC RXC0

#define UCSRB UCSR0B
#define RXEN RXEN0
#define TXEN TXEN0
#define RXCIE RXCIE0

#define UCSRC UCSR0C
#define URSEL 
#define UCSZ0 UCSZ00
#define UCSZ1 UCSZ01
#define UCSRC_SELECT 1 
#else
#define UCSRC_SELECT (1 << URSEL)
#define UBRRH UBRR0H
#define UBRRL UBRR0L

#endif
 
#ifndef USART_RXC_vect
#if defined(UART0_RX_vect)
#define USART_RXC_vect UART0_RX_vect
#elif defined(UART_RX_vect)
#define USART_RXC_vect UART_RX_vect
#elif defined(USART0_RX_vect)
#define USART_RXC_vect USART0_RX_vect
#elif defined(USART_RX_vect)
#define USART_RXC_vect USART_RX_vect
#elif defined(USART0_RXC_vect)
#define USART_RXC_vect USART0_RXC_vect
#elif defined(USART_RXC_vect)
#define USART_RXC_vect USART_RXC_vect
#else
#error "Uart receive complete interrupt not defined!"
#endif
#endif
 
extern volatile unsigned char usart_timeout;


void USART_Init(unsigned int UBRR_N, unsigned char DBR) {

 UBRRH=UBRR_N>>8;
 UBRRL=UBRR_N&255;
 // UBRRL = DBR; 

    /* set frame format: 8 bit, no parity, 1 bit */
    UCSRC = UCSRC_SELECT | (1 << UCSZ1) | (3 << UCSZ0);
    /* enable serial receiver and transmitter */
    UCSRB = (1 << RXEN) | (1 << TXEN); 
 
// UCSRB=(1<<RXEN)|(1<<TXEN);
// UCSRC=(1<<URSEL)|(3<<UCSZ);
  
 if (DBR) UCSRA|=(1<<U2X0);
 else UCSRA&=(unsigned char)~(1<<U2X0);
}


void USART_Transmit(unsigned char data) {

 while(!(UCSR0A&(1<<UDRE)));
 UDR=data;

}


unsigned char USART_Receive(void) {

 usart_timeout=180;
 while(!(UCSRA&(1<<RXC))) if(!usart_timeout) return 0;

 return UDR;

}


void USART_Flush(void) {

unsigned char dummy;

 while(UCSRA&(1<<RXC)) dummy=UDR;

}


void USART_pstr_P(const char *addr)
{
 char c;
 while((c=pgm_read_byte(addr++)))
  USART_putchar(c);
}
